@@ -44,8 +44,9 @@ def plot_convergence(
         ax.scatter(
             iter_data, conv_data, color=col_1, marker="o", alpha=0.4)
 
-        # Now plot a line an mark only the first and the last element
-        markevery = (0, len(conv_frame) - 1)
+        # Now plot a line and mark only the first and the last element
+        markevery = (0, max(len(conv_frame) - 1, 1))
+       
         ax.plot(
             iter_data, conv_data, color=col_0, marker="o",
             linestyle="-", markevery=markevery,
