@@ -4,6 +4,7 @@ import numpy as np
 import copy
 import uuid
 import pint
+from concurrent.futures import ThreadPoolExecutor
 import threading
 from simunetcore import exception
 
@@ -24,7 +25,7 @@ class Waveform(object):
         models : dict
             The dictionary of models.
         edges : list
-            The edges (adjecencies) of the flowsheet.
+            The edges (adjacencies) of the flowsheet.
         var_map : list 
             The variable map of the `edges`.
         simulation_id : uuid, optional
@@ -399,9 +400,13 @@ class Waveform(object):
         """
 
         try:
-            # Get model and client from dictionary
+            # Get model from dictionary
             model = self.models[model_id]
-            kwargs = self.compute_kwargs[model_id] if model_id in self.compute_kwargs else {}
+            
+            # Get compute args for the model
+            kwargs = {} 
+            if model_id in self.compute_kwargs:
+                kwargs = self.compute_kwargs[model_id]
 
             # Perform computation
             if model.threadsafe:
@@ -477,21 +482,11 @@ class Waveform(object):
                 for model_id in self.order:
                     # Prepare model coupling
                     self.prepare_model_coupling(old_state, model_id, t_frame)
-                for model_id in self.order:
-                    # Create task list
-                    tasks.append(
-                        threading.Thread(
-                            target=self.compute_model,
-                            args=(model_id,)))
-                    
-                # Start all threads    
-                for task in tasks:
-                    task.start()
-                
-                # Wait for completion
-                for task in tasks:
-                    task.join()
-                                     
+
+                with ThreadPoolExecutor() as executor:
+                    for model_id in self.order:
+                        executor.submit(self.compute_model, model_id)
+                                    
             # Compute convergence for every model and
             # all of its output variables
             convergence_iter = self.compute_convergence(
@@ -516,10 +511,10 @@ class Waveform(object):
     
     def compute(
             self, t_start, t_end, t_out=None, frame_length=0, 
-            min_frame_length=0, max_frame_length=0, max_iter_frame=7, 
-            points_per_frame=1001, max_iter=5000, iter_threshold=1, 
-            alpha=1.5, beta=0.2,epsilon=0.0001, step_type="gauss-seidel",
-            verbose = False):
+            min_frame_length=0, max_frame_length=0, max_iter_frame=0, 
+            points_per_frame=101, max_iter=5000, iter_threshold=1, 
+            alpha=1.5, beta=0.25, epsilon=0.001, step_type="gauss-seidel",
+            verbose = True):
         """
         Computes the waveform relaxation. Raises a ComputeError if
         something went wrong. Calls the callbacks 
@@ -537,19 +532,19 @@ class Waveform(object):
         t_out : list, optional
             The time frame for the result ouput. Default = None. 
         frame_length : float, optional
-            The frame length of thte inital segment.  Default = 0.
+            The frame length of thte inital segment. Default = 0.
         min_frame_length : float, optional
             The minimum frame length. The segments can't get smaller
-            than this value. Default = 0. 
+            than this value. Default = 0 (Automatically adjusted). 
         max_frame_length : float, optional
             The maximum frame length. The segments can't get larger
-            than this value. Default = 0.
+            than this value. Default = 0 (Automatically adjusted).
         max_iter_frame : int, optional
             The maximum number of iterations that are allowed.
-            Default = 7. 
+            Default = 0 (Automatically adjusted). 
         points_per_frame : int, optional
             The number of points per segment.
-            Default = 1001. 
+            Default = 201. 
         max_iter : int, optional
             The maximum number of overall iterations.
             Default = 5000.
@@ -563,7 +558,7 @@ class Waveform(object):
         beta : float, optional
             The contraction factor. A segment shrinks by this factor if the
             conditions are fullfilled.
-            Default = 0.2.
+            Default = 0.25.
         epsilon : float, optional
             The maximum allowed tolerance, used for the convergence test.
             Default = 0.0001.
@@ -572,7 +567,7 @@ class Waveform(object):
             Default = 'gauss-seidel'.
         verbose : bool, optional
             If set to True, a bunch of state logs are printed on the console.
-            Default = False.
+            Default = True.
 
         Raises
         ------
@@ -612,15 +607,18 @@ class Waveform(object):
         self.beta = beta
         self.step_type = step_type
         
+        # Set minimum length of frame
+        if self.max_iter_frame <= 0:
+            self.max_iter_frame = len(self.models) + 4
         # Adjust length of frames if frame_length was not set
         if self.frame_length > t_end - t_start or self.frame_length <= 0:
-            self.frame_length = 0.05 * (t_end - t_start)
+            self.frame_length = 0.02 * (t_end - t_start)
         # Set minimum length of frame
         if self.min_frame_length <= 0:
-            self.min_frame_length = 0.05 * self.frame_length
+            self.min_frame_length = self.frame_length * self.beta**3
         # Set maximum length of frame
         if self.max_frame_length <= 0:
-            self.max_frame_length = 5.0 * self.frame_length
+            self.max_frame_length = self.frame_length * self.alpha**3
         # Frame state variable
         frame_state = ""
 
@@ -630,19 +628,20 @@ class Waveform(object):
 
         # print job properties
         if verbose:
-            msg = "Waveform '{}' using '{}' base steps.\n".format(self.simulation_id, step_type)
-            msg += "    t_start = {}\n".format(t_start)
-            msg += "    t_end = {}\n".format(t_end)
-            msg += "    frame_length = {}\n".format(frame_length)
-            msg += "    min_frame_length = {}\n".format(min_frame_length)
-            msg += "    max_frame_length = {}\n".format(max_frame_length)
-            msg += "    max_iter_frame  = {}\n".format(max_iter_frame)
-            msg += "    points_per_frame = {}\n".format(points_per_frame)
-            msg += "    max_iter  = {}\n".format(max_iter)
-            msg += "    iter_threshold = {}\n".format(iter_threshold)
-            msg += "    alpha  = {}\n".format(alpha)
-            msg += "    beta  = {}\n".format(beta)
-            msg += "    epsilon = {}".format(epsilon)
+            msg = "Starting '{}'\n".format(self.simulation_id)
+            msg += "    t_start = {:.3E}\n".format(self.t_start)
+            msg += "    t_end = {:.3E}\n".format(self.t_end)
+            msg += "    frame_length = {:.3E}\n".format(self.frame_length)
+            msg += "    min_frame_length = {:.3E}\n".format(self.min_frame_length)
+            msg += "    max_frame_length = {:.3E}\n".format(self.max_frame_length)
+            msg += "    max_iter_frame = {:d}\n".format(self.max_iter_frame)
+            msg += "    points_per_frame = {:d}\n".format(self.points_per_frame)
+            msg += "    max_iter = {:d}\n".format(self.max_iter)
+            msg += "    iter_threshold = {:d}\n".format(self.iter_threshold)
+            msg += "    alpha = {:.3E}\n".format(self.alpha)
+            msg += "    beta = {:.3E}\n".format(self.beta)
+            msg += "    epsilon = {:.3E}\n".format(self.epsilon)
+            msg += "    step_type = '{}'".format(self.step_type)
             print(msg)
             
         # Prepare the frames list holding the results for each frame
